Package: lmerTest
Type: Package
Title: Tests in Linear Mixed Effects Models
Version: 3.2-1
Authors@R: c(person("Alexandra", "Kuznetsova", role = c("aut")),
                   person("Per", "Bruun Brockhoff", role = c("aut", "ths")),
                   person("Rune", "Haubo Bojesen Christensen", role =  c("aut", "cre"),
                   email = "Rune.Haubo@gmail.com", 
                   comment = c(ORCID = "0000-0002-4494-3399")),
                   person("Sofie", "Pødenphant Jensen", role=c("ctb")))
Depends: R (>= 3.2.5), lme4 (>= 1.1-10), stats, methods
Imports: numDeriv, MASS, ggplot2, reformulas
Suggests: pbkrtest (>= 0.4-3), tools
Description: Provides p-values in type I, II or III anova and summary tables
    for lmer model fits (cf. lme4) via Satterthwaite's degrees of freedom method. A
    Kenward-Roger method is also available via the pbkrtest package. Model selection
    methods include step, drop1 and anova-like tables for random effects (ranova).
    Methods for Least-Square means (LS-means) and tests of linear contrasts of fixed
    effects are also available.
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
URL: https://github.com/runehaubo/lmerTestR
BugReports: https://github.com/runehaubo/lmerTestR/issues
RoxygenNote: 7.3.3
Collate: 'anova_contrasts.R' 'contest.R' 'contrast_utils.R'
        'data_documentation.R' 'drop1.R' 'estimability.R' 'legacy.R'
        'lmer.R' 'lmerTest.R' 'lmer_anova.R' 'lmer_summary.R'
        'ls_means.R' 'ranova.R' 'step.R' 'terms_utils.R' 'utils.R'
NeedsCompilation: no
Packaged: 2026-03-05 07:48:42 UTC; runechristensen
Author: Alexandra Kuznetsova [aut],
  Per Bruun Brockhoff [aut, ths],
  Rune Haubo Bojesen Christensen [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4494-3399>),
  Sofie Pødenphant Jensen [ctb]
Maintainer: Rune Haubo Bojesen Christensen <Rune.Haubo@gmail.com>
Repository: CRAN
Date/Publication: 2026-03-05 12:20:02 UTC
Built: R 4.6.0; ; 2026-04-27 21:57:08 UTC; unix
