# Generated by roxygen2: do not edit by hand


if(getRversion() >= "3.3.0") {
  importFrom("stats", sigma)
} else {
  export(sigma)
}

S3method(anova,lmerModLmerTest)
S3method(anova,merModLmerTest)
S3method(as.data.frame,ls_means)
S3method(contest,lmerMod)
S3method(contest,lmerModLmerTest)
S3method(contest1D,lmerMod)
S3method(contest1D,lmerModLmerTest)
S3method(contestMD,lmerMod)
S3method(contestMD,lmerModLmerTest)
S3method(difflsmeans,lmerModLmerTest)
S3method(difflsmeans,merModLmerTest)
S3method(drop1,lmerModLmerTest)
S3method(drop1,merModLmerTest)
S3method(get_model,step_list)
S3method(ls_means,lmerModLmerTest)
S3method(ls_means,merModLmerTest)
S3method(lsmeansLT,lmerModLmerTest)
S3method(lsmeansLT,merModLmerTest)
S3method(plot,ls_means)
S3method(plot,step_list)
S3method(print,ls_means)
S3method(print,step_list)
S3method(show_tests,anova)
S3method(show_tests,default)
S3method(show_tests,ls_means)
S3method(step,default)
S3method(step,lmerModLmerTest)
S3method(step,merModLmerTest)
S3method(summary,lmerModLmerTest)
S3method(summary,merModLmerTest)
S3method(update,lmerModLmerTest)
export(as_lmerModLmerTest)
export(calcSatterth)
export(contest)
export(contest1D)
export(contestMD)
export(difflsmeans)
export(get_model)
export(lmer)
export(lmerModLmerTest)
export(ls_means)
export(lsmeansLT)
export(merModLmerTest)
export(rand)
export(ranova)
export(show_tests)
export(step)
exportClasses(lmerModLmerTest)
exportClasses(merModLmerTest)
importClassesFrom(lme4,lmerMod)
importFrom(MASS,fractions)
importFrom(MASS,ginv)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,rel)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(graphics,plot)
importFrom(lme4,fixef)
importFrom(lme4,getME)
importFrom(lme4,lmerControl)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,signature)
importFrom(numDeriv,hessian)
importFrom(numDeriv,jacobian)
importFrom(reformulas,findbars_x)
importFrom(reformulas,nobars)
importFrom(reformulas,splitForm)
importFrom(stats,.getXlevels)
importFrom(stats,.lm.fit)
importFrom(stats,anova)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,drop.scope)
importFrom(stats,drop1)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,lm.fit)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,nobs)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,printCoefmat)
importFrom(stats,pt)
importFrom(stats,qt)
importFrom(stats,resid)
importFrom(stats,setNames)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,vcov)
importFrom(utils,as.roman)
importFrom(utils,combn)
importFrom(utils,getFromNamespace)
importFrom(utils,packageVersion)
